@@ -64,16 +64,18 @@
 #include "util.h"
 #include "LogFactory.h"
 #include "DownloadContext.h"
+#include "wallclock.h"
+#include "NameResolver.h"
 
 namespace aria2 {
 
-AbstractCommand::AbstractCommand(int32_t cuid,
+AbstractCommand::AbstractCommand(cuid_t cuid,
                                  const SharedHandle<Request>& req,
                                  const SharedHandle<FileEntry>& fileEntry,
                                  RequestGroup* requestGroup,
                                  DownloadEngine* e,
                                  const SocketHandle& s):
-  Command(cuid), _requestGroup(requestGroup),
+  Command(cuid), checkPoint(global::wallclock), _requestGroup(requestGroup),
   req(req), _fileEntry(fileEntry), e(e), socket(s),
   checkSocketIsReadable(false), checkSocketIsWritable(false),
   nameResolverCheck(false)
@@ -98,25 +100,34 @@ AbstractCommand::~AbstractCommand() {
 
 bool AbstractCommand::execute() {
   if(logger->debug()) {
-    logger->debug("CUID#%d - socket: read:%d, write:%d, hup:%d, err:%d",
-                  cuid, _readEvent, _writeEvent, _hupEvent, _errorEvent);
+    logger->debug("CUID#%s - socket: read:%d, write:%d, hup:%d, err:%d",
+                  util::itos(cuid).c_str(), _readEvent, _writeEvent, _hupEvent,
+                  _errorEvent);
   }
   try {
     if(_requestGroup->downloadFinished() || _requestGroup->isHaltRequested()) {
-      //logger->debug("CUID#%d - finished.", cuid);
       return true;
     }
+    if(!req.isNull() && req->removalRequested()) {
+      if(logger->debug()) {
+        logger->debug
+          ("CUID#%s - Discard original URI=%s because it is requested.",
+           util::itos(cuid).c_str(), req->getUri().c_str());
+      }
+      return prepareForRetry(0);
+    }
     // TODO it is not needed to check other PeerStats every time.
     // Find faster Request when no segment is available.
     if(!req.isNull() && _fileEntry->countPooledRequest() > 0 &&
        !_requestGroup->getPieceStorage()->hasMissingUnusedPiece()) {
       SharedHandle<Request> fasterRequest = _fileEntry->findFasterRequest(req);
       if(!fasterRequest.isNull()) {
-        logger->info("CUID#%d - Use faster Request hostname=%s, port=%u",
-                     cuid,
-                     fasterRequest->getHost().c_str(),
-                     fasterRequest->getPort());
-
+        if(logger->info()) {
+          logger->info("CUID#%s - Use faster Request hostname=%s, port=%u",
+                       util::itos(cuid).c_str(),
+                       fasterRequest->getHost().c_str(),
+                       fasterRequest->getPort());
+        }
         // Cancel current Request object and use faster one.
         _fileEntry->removeRequest(req);
         Command* command =
@@ -134,10 +145,19 @@ bool AbstractCommand::execute() {
        (nameResolverCheck && nameResolveFinished()) ||
 #endif // ENABLE_ASYNC_DNS
        (!checkSocketIsReadable && !checkSocketIsWritable && !nameResolverCheck)) {
-      checkPoint.reset();
+      checkPoint = global::wallclock;
       if(!_requestGroup->getPieceStorage().isNull()) {
         _segments.clear();
         _requestGroup->getSegmentMan()->getInFlightSegment(_segments, cuid);
+        if(!req.isNull() && _segments.empty()) {
+          // This command previously has assigned segments, but it is
+          // canceled. So discard current request chain.
+          if(logger->debug()) {
+            logger->debug("CUID#%s - It seems previously assigned segments are"
+                          "canceled. Restart.", util::itos(cuid).c_str());
+          }
+          return prepareForRetry(0);
+        }
         if(req.isNull() || req->getMaxPipelinedRequest() == 1 ||
            _requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
           if(_segments.empty()) {
@@ -149,10 +169,15 @@ bool AbstractCommand::execute() {
           }
           if(_segments.empty()) {
             // TODO socket could be pooled here if pipelining is enabled...
-            logger->info(MSG_NO_SEGMENT_AVAILABLE, cuid);
+            if(logger->info()) {
+              logger->info(MSG_NO_SEGMENT_AVAILABLE, util::itos(cuid).c_str());
+            }
             // When all segments are ignored in SegmentMan, there are
             // no URIs available, so don't retry.
             if(_requestGroup->getSegmentMan()->allSegmentsIgnored()) {
+              if(logger->debug()) {
+                logger->debug("All segments are ignored.");
+              }
               return true;
             } else {
               return prepareForRetry(1);
@@ -175,7 +200,7 @@ bool AbstractCommand::execute() {
         (StringFormat(MSG_NETWORK_PROBLEM,
                       socket->getSocketError().c_str()).str());
     } else {
-      if(checkPoint.elapsed(timeout)) {
+      if(checkPoint.difference(global::wallclock) >= timeout) {
         // timeout triggers ServerStat error state.
 
         SharedHandle<ServerStat> ss =
@@ -196,30 +221,42 @@ bool AbstractCommand::execute() {
     } else {
       logger->error(MSG_DOWNLOAD_ABORTED,
                     DL_ABORT_EX2(StringFormat
-                                 ("URI=%s", req->getCurrentUrl().c_str()).str(),err),
-                    cuid, req->getUrl().c_str());
-      _fileEntry->addURIResult(req->getUrl(), err.getCode());
-      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+                                 ("URI=%s", req->getCurrentUri().c_str()).str(),err),
+                    util::itos(cuid).c_str(), req->getUri().c_str());
+      _fileEntry->addURIResult(req->getUri(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUri(), err.getCode());
+      if(err.getCode() == downloadresultcode::CANNOT_RESUME) {
+        _requestGroup->increaseResumeFailureCount();
+      }
     }
     onAbort();
     tryReserved();
     return true;
   } catch(DlRetryEx& err) {
     assert(!req.isNull());
-    logger->info(MSG_RESTARTING_DOWNLOAD,
-                 DL_RETRY_EX2(StringFormat
-                              ("URI=%s", req->getCurrentUrl().c_str()).str(),err),
-                 cuid, req->getUrl().c_str());
+    if(logger->info()) {
+      logger->info(MSG_RESTARTING_DOWNLOAD,
+                   DL_RETRY_EX2(StringFormat
+                                ("URI=%s", req->getCurrentUri().c_str()).str(),
+                                err),
+                   util::itos(cuid).c_str(), req->getUri().c_str());
+    }
     req->addTryCount();
     req->resetRedirectCount();
     const unsigned int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
     bool isAbort = maxTries != 0 && req->getTryCount() >= maxTries;
     if(isAbort) {
+      if(logger->info()) {
+        logger->info(MSG_MAX_TRY, util::itos(cuid).c_str(), req->getTryCount());
+      }
+      logger->error(MSG_DOWNLOAD_ABORTED, err, util::itos(cuid).c_str(),
+                    req->getUri().c_str());
+      _fileEntry->addURIResult(req->getUri(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUri(), err.getCode());
+      if(err.getCode() == downloadresultcode::CANNOT_RESUME) {
+        _requestGroup->increaseResumeFailureCount();
+      }
       onAbort();
-      logger->info(MSG_MAX_TRY, cuid, req->getTryCount());
-      logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
-      _fileEntry->addURIResult(req->getUrl(), err.getCode());
-      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
       tryReserved();
       return true;
     } else {
@@ -228,8 +265,8 @@ bool AbstractCommand::execute() {
   } catch(DownloadFailureException& err) {
     logger->error(EX_EXCEPTION_CAUGHT, err);
     if(!req.isNull()) {
-      _fileEntry->addURIResult(req->getUrl(), err.getCode());
-      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+      _fileEntry->addURIResult(req->getUri(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUri(), err.getCode());
     }
     _requestGroup->setHaltRequested(true);
     return true;
@@ -245,17 +282,19 @@ void AbstractCommand::tryReserved() {
     // can assume that there are no in-flight request object.
     if(entry->getLength() == 0 && entry->getRemainingUris().empty()) {
       if(logger->debug()) {
-        logger->debug("CUID#%d - Not trying next request."
+        logger->debug("CUID#%s - Not trying next request."
                       " No reserved/pooled request is remaining and"
-                      " total length is still unknown.", cuid);
+                      " total length is still unknown.",
+                      util::itos(cuid).c_str());
       }
       return;
     }
   }
   if(logger->debug()) {
-    logger->debug("CUID#%d - Trying reserved/pooled request.", cuid);
+    logger->debug("CUID#%s - Trying reserved/pooled request.",
+                  util::itos(cuid).c_str());
   }
-  Commands commands;
+  std::vector<Command*> commands;
   _requestGroup->createNextCommand(commands, e, 1);
   e->setNoWait(true);
   e->addCommand(commands);
@@ -268,8 +307,8 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
   if(!req.isNull()) {
     _fileEntry->poolRequest(req);
     if(logger->debug()) {
-      logger->debug("CUID#%d - Pooling request URI=%s",
-                    cuid, req->getUrl().c_str());
+      logger->debug("CUID#%s - Pooling request URI=%s",
+                    util::itos(cuid).c_str(), req->getUri().c_str());
     }
     if(!_requestGroup->getSegmentMan().isNull()) {
       _requestGroup->getSegmentMan()->recognizeSegmentFor(_fileEntry);
@@ -293,14 +332,59 @@ void AbstractCommand::onAbort() {
     // TODO This might be a problem if the failure is caused by proxy.
     e->_requestGroupMan->getOrCreateServerStat(req->getHost(),
                                                req->getProtocol())->setError();
-    _fileEntry->removeIdenticalURI(req->getUrl());
+    _fileEntry->removeIdenticalURI(req->getUri());
     _fileEntry->removeRequest(req);
   }
   if(logger->debug()) {
-    logger->debug("CUID#%d - Aborting download", cuid);
+    logger->debug("CUID#%s - Aborting download", util::itos(cuid).c_str());
   }
   if(!_requestGroup->getPieceStorage().isNull()) {
-    _requestGroup->getSegmentMan()->cancelSegment(cuid);
+    SharedHandle<SegmentMan> segmentMan = _requestGroup->getSegmentMan();
+    segmentMan->cancelSegment(cuid);
+    // Don't do following process if BitTorrent is involved or files
+    // in DownloadContext is more than 1. The latter condition is
+    // limitation of current implementation.
+    if(!getOption()->getAsBool(PREF_ALWAYS_RESUME) &&
+       !_fileEntry.isNull() &&
+       segmentMan->calculateSessionDownloadLength() == 0 &&
+       !_requestGroup->p2pInvolved() &&
+       _requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
+      const int maxTries = getOption()->getAsInt(PREF_MAX_RESUME_FAILURE_TRIES);
+      if((maxTries > 0 && _requestGroup->getResumeFailureCount() >= maxTries)||
+         _fileEntry->emptyRequestUri()) {
+        // Local file exists, but given servers(or at least contacted
+        // ones) doesn't support resume. Let's restart download from
+        // scratch.
+        logger->notice("CUID#%s - Failed to resume download."
+                       " Download from scratch.",
+                       util::itos(cuid).c_str());
+        if(logger->debug()) {
+          logger->debug("CUID#%s - Gathering URIs that has CANNOT_RESUME error",
+                        util::itos(cuid).c_str());
+        }
+        // Set PREF_ALWAYS_RESUME to V_TRUE to avoid repeating this
+        // process.
+        getOption()->put(PREF_ALWAYS_RESUME, V_TRUE);
+        std::deque<URIResult> res;
+        _fileEntry->extractURIResult(res, downloadresultcode::CANNOT_RESUME);
+        if(!res.empty()) {
+          segmentMan->cancelAllSegments();
+          segmentMan->eraseSegmentWrittenLengthMemo();
+          _requestGroup->getPieceStorage()->markPiecesDone(0);
+          std::vector<std::string> uris;
+          uris.reserve(res.size());
+          std::transform(res.begin(), res.end(), std::back_inserter(uris),
+                         std::mem_fun_ref(&URIResult::getURI));
+          if(logger->debug()) {
+            logger->debug("CUID#%s - %lu URIs found.",
+                          util::itos(cuid).c_str(),
+                          static_cast<unsigned long int>(uris.size()));
+          }
+          _fileEntry->addUris(uris.begin(), uris.end());
+          segmentMan->recognizeSegmentFor(_fileEntry);
+        }
+      }
+    }
   }
 }
 
@@ -409,7 +493,7 @@ static bool isProxyRequest
 (const std::string& protocol, const SharedHandle<Option>& option)
 {
   const std::string& proxyUri = getProxyUri(protocol, option);
-  return !proxyUri.empty() && Request().setUrl(proxyUri);
+  return !proxyUri.empty() && Request().setUri(proxyUri);
 }
 
 class DomainMatch {
@@ -436,9 +520,37 @@ static bool inNoProxy(const SharedHandle<Request>& req,
   if(entries.empty()) {
     return false;
   }
-  return
-    std::find_if(entries.begin(), entries.end(),
-                 DomainMatch(A2STR::DOT_C+req->getHost())) != entries.end();
+  DomainMatch domainMatch(A2STR::DOT_C+req->getHost());
+  for(std::vector<std::string>::const_iterator i = entries.begin(),
+        eoi = entries.end(); i != eoi; ++i) {
+    std::string::size_type slashpos = (*i).find('/');
+    if(slashpos == std::string::npos) {
+      if(util::isNumericHost(*i)) {
+        if(req->getHost() == *i) {
+          return true;
+        }
+      } else if(domainMatch(*i)) {
+        return true;
+      }
+    } else {
+      if(!util::isNumericHost(req->getHost())) {
+        // TODO We don't resolve hostname here.  More complete
+        // implementation is that we should first resolve
+        // hostname(which may result in several IP addresses) and
+        // evaluates against all of them
+        continue;
+      }
+      std::string ip = (*i).substr(0, slashpos);
+      uint32_t bits;
+      if(!util::parseUIntNoThrow(bits, (*i).substr(slashpos+1))) {
+        continue;
+      }
+      if(util::inSameCidrBlock(ip, req->getHost(), bits)) {
+        return true;
+      }
+    }
+  }
+  return false;
 }
 
 bool AbstractCommand::isProxyDefined() const
@@ -456,13 +568,14 @@ SharedHandle<Request> AbstractCommand::createProxyRequest() const
   std::string proxy = getProxyUri(req->getProtocol(), getOption());
   if(!proxy.empty()) {
     proxyRequest.reset(new Request());
-    if(proxyRequest->setUrl(proxy)) {
+    if(proxyRequest->setUri(proxy)) {
       if(logger->debug()) {
-        logger->debug("CUID#%d - Using proxy", cuid);
+        logger->debug("CUID#%s - Using proxy", util::itos(cuid).c_str());
       }
     } else {
       if(logger->debug()) {
-        logger->debug("CUID#%d - Failed to parse proxy string", cuid);
+        logger->debug("CUID#%s - Failed to parse proxy string",
+                      util::itos(cuid).c_str());
       }
       proxyRequest.reset();
     }
@@ -480,7 +593,10 @@ bool AbstractCommand::isAsyncNameResolverInitialized() const
 void AbstractCommand::initAsyncNameResolver(const std::string& hostname)
 {
   _asyncNameResolver.reset(new AsyncNameResolver());
-  logger->info(MSG_RESOLVING_HOSTNAME, cuid, hostname.c_str());
+  if(logger->info()) {
+    logger->info(MSG_RESOLVING_HOSTNAME,
+                 util::itos(cuid).c_str(), hostname.c_str());
+  }
   _asyncNameResolver->resolve(hostname);
   setNameResolverCheck(_asyncNameResolver);
 }
@@ -489,13 +605,16 @@ bool AbstractCommand::asyncResolveHostname()
 {
   switch(_asyncNameResolver->getStatus()) {
   case AsyncNameResolver::STATUS_SUCCESS:
+    disableNameResolverCheck(_asyncNameResolver);
     return true;
   case AsyncNameResolver::STATUS_ERROR:
+    disableNameResolverCheck(_asyncNameResolver);
     if(!isProxyRequest(req->getProtocol(), getOption())) {
       e->_requestGroupMan->getOrCreateServerStat
         (req->getHost(), req->getProtocol())->setError();
     }
-    throw DL_ABORT_EX(StringFormat(MSG_NAME_RESOLUTION_FAILED, cuid,
+    throw DL_ABORT_EX(StringFormat(MSG_NAME_RESOLUTION_FAILED,
+                                   util::itos(cuid).c_str(),
                                    _asyncNameResolver->getHostname().c_str(),
                                    _asyncNameResolver->getError().c_str()).str());
   default:
@@ -503,7 +622,7 @@ bool AbstractCommand::asyncResolveHostname()
   }
 }
 
-const std::deque<std::string>& AbstractCommand::getResolvedAddresses()
+const std::vector<std::string>& AbstractCommand::getResolvedAddresses()
 {
   return _asyncNameResolver->getResolvedAddresses();
 }
@@ -531,12 +650,59 @@ bool AbstractCommand::nameResolveFinished() const {
 }
 #endif // ENABLE_ASYNC_DNS
 
+std::string AbstractCommand::resolveHostname
+(std::vector<std::string>& addrs, const std::string& hostname, uint16_t port)
+{
+  e->findAllCachedIPAddresses(std::back_inserter(addrs), hostname, port);
+  std::string ipaddr;
+  if(addrs.empty()) {
+#ifdef ENABLE_ASYNC_DNS
+    if(getOption()->getAsBool(PREF_ASYNC_DNS)) {
+      if(!isAsyncNameResolverInitialized()) {
+        initAsyncNameResolver(hostname);
+      }
+      if(asyncResolveHostname()) {
+        addrs = getResolvedAddresses();
+      } else {
+        return A2STR::NIL;
+      }
+    } else
+#endif // ENABLE_ASYNC_DNS
+      {
+        NameResolver res;
+        res.setSocktype(SOCK_STREAM);
+        if(e->option->getAsBool(PREF_DISABLE_IPV6)) {
+          res.setFamily(AF_INET);
+        }
+        res.resolve(addrs, hostname);
+      }
+    if(logger->info()) {
+      logger->info(MSG_NAME_RESOLUTION_COMPLETE, util::itos(cuid).c_str(),
+                   hostname.c_str(),
+                   strjoin(addrs.begin(), addrs.end(), ", ").c_str());
+    }
+    for(std::vector<std::string>::const_iterator i = addrs.begin(),
+          eoi = addrs.end(); i != eoi; ++i) {
+      e->cacheIPAddress(hostname, *i, port);
+    }
+    ipaddr = e->findCachedIPAddress(hostname, port);
+  } else {
+    ipaddr = addrs.front();
+    if(logger->info()) {
+      logger->info(MSG_DNS_CACHE_HIT,
+                   util::itos(cuid).c_str(), hostname.c_str(),
+                   strjoin(addrs.begin(), addrs.end(), ", ").c_str());
+    }
+  }
+  return ipaddr;
+}
+
 void AbstractCommand::prepareForNextAction(Command* nextCommand)
 {
-  CheckIntegrityEntryHandle entry
+  SharedHandle<CheckIntegrityEntry> entry
     (new StreamCheckIntegrityEntry(_requestGroup, nextCommand));
 
-  std::deque<Command*> commands;
+  std::vector<Command*> commands;
   try {
     _requestGroup->processCheckIntegrityEntry(commands, entry, e);
   } catch(RecoverableException& e) {
@@ -559,8 +725,11 @@ bool AbstractCommand::checkIfConnectionEstablished
       // See also InitiateConnectionCommand::executeInternal()
       e->markBadIPAddress(connectedHostname, connectedAddr, connectedPort);
       if(!e->findCachedIPAddress(connectedHostname, connectedPort).empty()) {
-        logger->info(MSG_CONNECT_FAILED_AND_RETRY,
-                     cuid, connectedAddr.c_str(), connectedPort);
+        if(logger->info()) {
+          logger->info(MSG_CONNECT_FAILED_AND_RETRY,
+                       util::itos(cuid).c_str(),
+                       connectedAddr.c_str(), connectedPort);
+        }
         Command* command =
           InitiateConnectionCommandFactory::createInitiateConnectionCommand
           (cuid, req, _fileEntry, _requestGroup, e);
