@@ -35,6 +35,7 @@
 #include "DefaultBtInteractive.h"
 
 #include <cstring>
+#include <vector>
 
 #include "prefs.h"
 #include "message.h"
@@ -71,6 +72,7 @@
 #include "bittorrent_helper.h"
 #include "UTMetadataRequestFactory.h"
 #include "UTMetadataRequestTracker.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
@@ -83,6 +85,12 @@ DefaultBtInteractive::DefaultBtInteractive
   _metadataGetMode(false),
   logger(LogFactory::getInstance()),
   allowedFastSetSize(10),
+  haveTimer(global::wallclock),
+  keepAliveTimer(global::wallclock),
+  floodingTimer(global::wallclock),
+  inactiveTimer(global::wallclock),
+  _pexTimer(global::wallclock),
+  _perSecTimer(global::wallclock),
   keepAliveInterval(120),
   _utPexEnabled(false),
   _dhtEnabled(false),
@@ -110,29 +118,49 @@ BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
             PEER_ID_LENGTH) == 0) {
     throw DL_ABORT_EX
       (StringFormat
-       ("CUID#%d - Drop connection from the same Peer ID", cuid).str());
+       ("CUID#%s - Drop connection from the same Peer ID",
+        util::itos(cuid).c_str()).str());
+  }
+  std::vector<SharedHandle<Peer> > activePeers;
+  _peerStorage->getActivePeers(activePeers);
+  for(std::vector<SharedHandle<Peer> >::const_iterator i = activePeers.begin(),
+        eoi = activePeers.end(); i != eoi; ++i) {
+    if(memcmp((*i)->getPeerId(), message->getPeerId(), PEER_ID_LENGTH) == 0) {
+      throw DL_ABORT_EX
+        (StringFormat
+         ("CUID#%s - Same Peer ID has been already seen.",
+          util::itos(cuid).c_str()).str());
+    }
   }
 
   peer->setPeerId(message->getPeerId());
     
   if(message->isFastExtensionSupported()) {
     peer->setFastExtensionEnabled(true);
-    logger->info(MSG_FAST_EXTENSION_ENABLED, cuid);
+    if(logger->info()) {
+      logger->info(MSG_FAST_EXTENSION_ENABLED, util::itos(cuid).c_str());
+    }
   }
   if(message->isExtendedMessagingEnabled()) {
     peer->setExtendedMessagingEnabled(true);
     if(!_utPexEnabled) {
       _extensionMessageRegistry->removeExtension("ut_pex");
     }
-    logger->info(MSG_EXTENDED_MESSAGING_ENABLED, cuid);
+    if(logger->info()) {
+      logger->info(MSG_EXTENDED_MESSAGING_ENABLED, util::itos(cuid).c_str());
+    }
   }
   if(message->isDHTEnabled()) {
     peer->setDHTEnabled(true);
-    logger->info(MSG_DHT_ENABLED_PEER, cuid);
+    if(logger->info()) {
+      logger->info(MSG_DHT_ENABLED_PEER, util::itos(cuid).c_str());
+    }
+  }
+  if(logger->info()) {
+    logger->info(MSG_RECEIVE_PEER_MESSAGE, util::itos(cuid).c_str(),
+                 peer->ipaddr.c_str(), peer->port,
+                 message->toString().c_str());
   }
-  logger->info(MSG_RECEIVE_PEER_MESSAGE, cuid,
-               peer->ipaddr.c_str(), peer->port,
-               message->toString().c_str());
   return message;
 }
 
@@ -141,11 +169,11 @@ BtMessageHandle DefaultBtInteractive::receiveAndSendHandshake() {
 }
 
 void DefaultBtInteractive::doPostHandshakeProcessing() {
-  // Set time 0 to haveCheckPoint to cache http/ftp download piece completion
-  haveCheckPoint.setTimeInSec(0);
-  keepAliveCheckPoint.reset();
-  floodingCheckPoint.reset();
-  _pexCheckPoint.setTimeInSec(0);
+  // Set time 0 to haveTimer to cache http/ftp download piece completion
+  haveTimer.reset(0);
+  keepAliveTimer = global::wallclock;
+  floodingTimer = global::wallclock;
+  _pexTimer.reset(0);
   if(peer->isExtendedMessagingEnabled()) {
     addHandshakeExtendedMessageToQueue();
   }
@@ -205,8 +233,8 @@ void DefaultBtInteractive::addAllowedFastMessageToQueue() {
                                _downloadContext->getNumPieces(),
                                bittorrent::getInfoHash(_downloadContext),
                                allowedFastSetSize);
-    for(std::vector<size_t>::const_iterator itr = fastSet.begin();
-        itr != fastSet.end(); ++itr) {
+    for(std::vector<size_t>::const_iterator itr = fastSet.begin(),
+          eoi = fastSet.end(); itr != eoi; ++itr) {
       dispatcher->addMessageToQueue
         (messageFactory->createAllowedFastMessage(*itr));
     }
@@ -226,9 +254,9 @@ void DefaultBtInteractive::decideChoking() {
 }
 
 void DefaultBtInteractive::checkHave() {
-  std::deque<size_t> indexes;
-  _pieceStorage->getAdvertisedPieceIndexes(indexes, cuid, haveCheckPoint);
-  haveCheckPoint.reset();
+  std::vector<size_t> indexes;
+  _pieceStorage->getAdvertisedPieceIndexes(indexes, cuid, haveTimer);
+  haveTimer = global::wallclock;
   if(indexes.size() >= 20) {
     if(peer->isFastExtensionEnabled() && _pieceStorage->allDownloadFinished()) {
       dispatcher->addMessageToQueue(messageFactory->createHaveAllMessage());
@@ -236,18 +264,18 @@ void DefaultBtInteractive::checkHave() {
       dispatcher->addMessageToQueue(messageFactory->createBitfieldMessage());
     }
   } else {
-    for(std::deque<size_t>::iterator itr = indexes.begin();
-        itr != indexes.end(); ++itr) {
+    for(std::vector<size_t>::const_iterator itr = indexes.begin(),
+          eoi = indexes.end(); itr != eoi; ++itr) {
       dispatcher->addMessageToQueue(messageFactory->createHaveMessage(*itr));
     }
   }
 }
 
 void DefaultBtInteractive::sendKeepAlive() {
-  if(keepAliveCheckPoint.elapsed(keepAliveInterval)) {
+  if(keepAliveTimer.difference(global::wallclock) >= keepAliveInterval) {
     dispatcher->addMessageToQueue(messageFactory->createKeepAliveMessage());
     dispatcher->sendMessages();
-    keepAliveCheckPoint.reset();
+    keepAliveTimer = global::wallclock;
   }
 }
 
@@ -264,9 +292,11 @@ size_t DefaultBtInteractive::receiveMessages() {
       break;
     }
     ++msgcount;
-    logger->info(MSG_RECEIVE_PEER_MESSAGE, cuid,
-                 peer->ipaddr.c_str(), peer->port,
-                 message->toString().c_str());
+    if(logger->info()) {
+      logger->info(MSG_RECEIVE_PEER_MESSAGE, util::itos(cuid).c_str(),
+                   peer->ipaddr.c_str(), peer->port,
+                   message->toString().c_str());
+    }
     message->doReceivedAction();
 
     switch(message->getId()) {
@@ -287,7 +317,7 @@ size_t DefaultBtInteractive::receiveMessages() {
       _peerStorage->updateTransferStatFor(peer);
       // pass through
     case BtRequestMessage::ID:
-      inactiveCheckPoint.reset();
+      inactiveTimer = global::wallclock;
       break;
     }
   }
@@ -303,7 +333,7 @@ void DefaultBtInteractive::decideInterest() {
   if(_pieceStorage->hasMissingPiece(peer)) {
     if(!peer->amInterested()) {
       if(logger->debug()) {
-        logger->debug(MSG_PEER_INTERESTED, cuid);
+        logger->debug(MSG_PEER_INTERESTED, util::itos(cuid).c_str());
       }
       dispatcher->
         addMessageToQueue(messageFactory->createInterestedMessage());
@@ -311,7 +341,7 @@ void DefaultBtInteractive::decideInterest() {
   } else {
     if(peer->amInterested()) {
       if(logger->debug()) {
-        logger->debug(MSG_PEER_NOT_INTERESTED, cuid);
+        logger->debug(MSG_PEER_NOT_INTERESTED, util::itos(cuid).c_str());
       }
       dispatcher->
         addMessageToQueue(messageFactory->createNotInterestedMessage());
@@ -326,7 +356,8 @@ void DefaultBtInteractive::fillPiece(size_t maxMissingBlock) {
 
     if(peer->peerChoking()) {
       if(peer->isFastExtensionEnabled()) {
-        std::deque<size_t> excludedIndexes;
+        std::vector<size_t> excludedIndexes;
+        excludedIndexes.reserve(btRequestFactory->countTargetPiece());
         btRequestFactory->getTargetPieceIndexes(excludedIndexes);
         while(numMissingBlock < maxMissingBlock) {
           SharedHandle<Piece> piece =
@@ -341,7 +372,8 @@ void DefaultBtInteractive::fillPiece(size_t maxMissingBlock) {
         }
       }
     } else {
-      std::deque<size_t> excludedIndexes;
+      std::vector<size_t> excludedIndexes;
+      excludedIndexes.reserve(btRequestFactory->countTargetPiece());
       btRequestFactory->getTargetPieceIndexes(excludedIndexes);
       while(numMissingBlock < maxMissingBlock) {
         SharedHandle<Piece> piece =
@@ -364,7 +396,8 @@ void DefaultBtInteractive::addRequests() {
     _maxOutstandingRequest <= dispatcher->countOutstandingRequest() ?
     0 : _maxOutstandingRequest-dispatcher->countOutstandingRequest();
   if(reqNumToCreate > 0) {
-    BtMessages requests;
+    std::vector<SharedHandle<BtMessage> > requests;
+    requests.reserve(reqNumToCreate);
     if(_pieceStorage->isEndGame()) {
       btRequestFactory->createRequestMessagesOnEndGame(requests,reqNumToCreate);
     } else {
@@ -379,8 +412,8 @@ void DefaultBtInteractive::cancelAllPiece() {
   if(_metadataGetMode && _downloadContext->getTotalLength() > 0) {
     std::vector<size_t> metadataRequests =
       _utMetadataRequestTracker->getAllTrackedIndex();
-    for(std::vector<size_t>::const_iterator i = metadataRequests.begin();
-        i != metadataRequests.end(); ++i) {
+    for(std::vector<size_t>::const_iterator i = metadataRequests.begin(),
+          eoi = metadataRequests.end(); i != eoi; ++i) {
       if(logger->debug()) {
         logger->debug("Cancel metadata: piece=%lu",
                       static_cast<unsigned long>(*i));
@@ -395,25 +428,27 @@ void DefaultBtInteractive::sendPendingMessage() {
 }
 
 void DefaultBtInteractive::detectMessageFlooding() {
-  if(floodingCheckPoint.elapsed(FLOODING_CHECK_INTERVAL)) {
+  if(floodingTimer.
+     difference(global::wallclock) >= FLOODING_CHECK_INTERVAL) {
     if(floodingStat.getChokeUnchokeCount() >= 2 ||
        floodingStat.getKeepAliveCount() >= 2) {
       throw DL_ABORT_EX(EX_FLOODING_DETECTED);
     } else {
       floodingStat.reset();
     }
-    floodingCheckPoint.reset();
+    floodingTimer = global::wallclock;
   }
 }
 
 void DefaultBtInteractive::checkActiveInteraction()
 {
+  time_t inactiveTime = inactiveTimer.difference(global::wallclock);
   // To allow aria2 to accept mutially interested peer, disconnect unintersted
   // peer.
   {
     const time_t interval = 30;
     if(!peer->amInterested() && !peer->peerInterested() &&
-       inactiveCheckPoint.elapsed(interval)) {
+       inactiveTime >= interval) {
       // TODO change the message
       throw DL_ABORT_EX
         (StringFormat("Disconnect peer because we are not interested each other"
@@ -425,7 +460,7 @@ void DefaultBtInteractive::checkActiveInteraction()
   // are disconnected in a certain time period.
   {
     const time_t interval = 60;
-    if(inactiveCheckPoint.elapsed(interval)) {
+    if(inactiveTime >= interval) {
       throw DL_ABORT_EX
         (StringFormat(EX_DROP_INACTIVE_CONNECTION, interval).str());
     }
@@ -434,13 +469,15 @@ void DefaultBtInteractive::checkActiveInteraction()
 
 void DefaultBtInteractive::addPeerExchangeMessage()
 {
-  if(_pexCheckPoint.elapsed(UTPexExtensionMessage::DEFAULT_INTERVAL)) {
+  if(_pexTimer.
+     difference(global::wallclock) >= UTPexExtensionMessage::DEFAULT_INTERVAL) {
     UTPexExtensionMessageHandle m
       (new UTPexExtensionMessage(peer->getExtensionMessageID("ut_pex")));
-    const Peers& peers = _peerStorage->getPeers();
+    const std::deque<SharedHandle<Peer> >& peers = _peerStorage->getPeers();
     {
       for(std::deque<SharedHandle<Peer> >::const_iterator i =
-            peers.begin(); i != peers.end() && !m->freshPeersAreFull(); ++i) {
+            peers.begin(), eoi = peers.end();
+          i != eoi && !m->freshPeersAreFull(); ++i) {
         if(peer->ipaddr != (*i)->ipaddr) {
           m->addFreshPeer(*i);
         }
@@ -448,7 +485,8 @@ void DefaultBtInteractive::addPeerExchangeMessage()
     }
     {
       for(std::deque<SharedHandle<Peer> >::const_reverse_iterator i =
-            peers.rbegin(); i != peers.rend() && !m->droppedPeersAreFull();
+            peers.rbegin(), eoi = peers.rend();
+          i != eoi && !m->droppedPeersAreFull();
           ++i) {
         if(peer->ipaddr != (*i)->ipaddr) {
           m->addDroppedPeer(*i);
@@ -457,7 +495,7 @@ void DefaultBtInteractive::addPeerExchangeMessage()
     }
     BtMessageHandle msg = messageFactory->createBtExtendedMessage(m);
     dispatcher->addMessageToQueue(msg);
-    _pexCheckPoint.reset();
+    _pexTimer = global::wallclock;
   }
 }
 
@@ -474,18 +512,18 @@ void DefaultBtInteractive::doInteractionProcessing() {
        _downloadContext->getTotalLength() > 0) {
       size_t num = _utMetadataRequestTracker->avail();
       if(num > 0) {
-        std::deque<SharedHandle<BtMessage> > requests;
+        std::vector<SharedHandle<BtMessage> > requests;
         _utMetadataRequestFactory->create(requests, num, _pieceStorage);
         dispatcher->addMessageToQueue(requests);
       }
-      if(_perSecCheckPoint.elapsed(1)) {
-        _perSecCheckPoint.reset();
+      if(_perSecTimer.difference(global::wallclock) >= 1) {
+        _perSecTimer = global::wallclock;
         // Drop timeout request after queuing message to give a chance
         // to other connection to request piece.
         std::vector<size_t> indexes =
           _utMetadataRequestTracker->removeTimeoutEntry();
-        for(std::vector<size_t>::const_iterator i = indexes.begin();
-            i != indexes.end(); ++i) {
+        for(std::vector<size_t>::const_iterator i = indexes.begin(),
+              eoi = indexes.end(); i != eoi; ++i) {
           _pieceStorage->cancelPiece(_pieceStorage->getPiece(*i));
         }
       }
@@ -501,8 +539,8 @@ void DefaultBtInteractive::doInteractionProcessing() {
 
     detectMessageFlooding();
 
-    if(_perSecCheckPoint.elapsed(1)) {
-      _perSecCheckPoint.reset();
+    if(_perSecTimer.difference(global::wallclock) >= 1) {
+      _perSecTimer = global::wallclock;
       dispatcher->checkRequestSlotAndDoNecessaryThing();
     }
     checkHave();
