@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -40,65 +40,66 @@
 #include <stdint.h>
 
 #include <string>
-#include <deque>
+#include <vector>
 
 #include "SharedHandle.h"
-#include "FileEntry.h"
+#include "DownloadResultCode.h"
+#include "RequestGroup.h"
+#include "Option.h"
+#include "MetadataInfo.h"
 
 namespace aria2 {
 
+class FileEntry;
+
 class DownloadResult
 {
 public:
-  // RESULT is used as an exit status code.
-  enum RESULT {
-    FINISHED = 0,
-    UNKNOWN_ERROR = 1,
-    TIME_OUT = 2,
-    RESOURCE_NOT_FOUND = 3,
-    MAX_FILE_NOT_FOUND = 4,
-    TOO_SLOW_DOWNLOAD_SPEED = 5,
-    NETWORK_PROBLEM = 6,
-    IN_PROGRESS = 7,
-  };
-
-  int32_t gid;
+  gid_t gid;
  
-  std::deque<SharedHandle<FileEntry> > fileEntries;
+  std::vector<SharedHandle<FileEntry> > fileEntries;
 
   bool inMemoryDownload;
 
-  uint64_t totalLength;
-
-  std::string uri;
-
-  size_t numUri;
-
   uint64_t sessionDownloadLength;
 
   // milliseconds
   int64_t sessionTime;
 
-  RESULT result;
+  downloadresultcode::RESULT result;
 
-  DownloadResult(int32_t gid,
-		 const std::deque<SharedHandle<FileEntry> >& fileEntries,
-		 bool inMemoryDownload,
-		 uint64_t totalLength,
-		 const std::string& uri,
-		 size_t numUri,
-		 uint64_t sessionDownloadLength,
-		 int64_t sessionTime,
-		 RESULT result):
+  // This field contains GIDs. See comment in
+  // RequestGroup.cc::_followedByGIDs.
+  std::vector<gid_t> followedBy;
+
+  // This field contains GID. See comment in
+  // RequestGroup.cc::_belongsToGID.
+  gid_t belongsTo;
+
+  SharedHandle<Option> option;
+
+  SharedHandle<MetadataInfo> metadataInfo;
+
+  DownloadResult(gid_t gid,
+                 const std::vector<SharedHandle<FileEntry> >& fileEntries,
+                 bool inMemoryDownload,
+                 uint64_t sessionDownloadLength,
+                 int64_t sessionTime,
+                 downloadresultcode::RESULT result,
+                 const std::vector<gid_t> followedBy,
+                 gid_t belongsTo,
+                 const SharedHandle<Option>& option,
+                 const SharedHandle<MetadataInfo>& metadataInfo):
     gid(gid),
     fileEntries(fileEntries),
     inMemoryDownload(inMemoryDownload),
-    totalLength(totalLength),
-    uri(uri),
-    numUri(numUri),
     sessionDownloadLength(sessionDownloadLength),
     sessionTime(sessionTime),
-    result(result) {}
+    result(result),
+    followedBy(followedBy),
+    belongsTo(belongsTo),
+    option(option),
+    metadataInfo(metadataInfo) {}
 };
 
 typedef SharedHandle<DownloadResult> DownloadResultHandle;
