@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -67,8 +67,8 @@ std::string HandshakeExtensionMessage::getPayload()
     dict["p"] = _tcpPort;
   }
   BDE extDict = BDE::dict();
-  for(std::map<std::string, uint8_t>::const_iterator itr = _extensions.begin();
-      itr != _extensions.end(); ++itr) {
+  for(std::map<std::string, uint8_t>::const_iterator itr = _extensions.begin(),
+        eoi = _extensions.end(); itr != eoi; ++itr) {
     const std::map<std::string, uint8_t>::value_type& vt = *itr;
     extDict[vt.first] = vt.second;
   }
@@ -83,7 +83,7 @@ std::string HandshakeExtensionMessage::toString() const
 {
   std::string s = getExtensionName();
   if(!_clientVersion.empty()) {
-    strappend(s, " client=", util::urlencode(_clientVersion));
+    strappend(s, " client=", util::percentEncode(_clientVersion));
   }
   if(_tcpPort > 0) {
     strappend(s, ", tcpPort=", util::uitos(_tcpPort));
@@ -91,8 +91,8 @@ std::string HandshakeExtensionMessage::toString() const
   if(_metadataSize) {
     strappend(s, ", metadataSize=", util::uitos(_metadataSize));
   }
-  for(std::map<std::string, uint8_t>::const_iterator itr = _extensions.begin();
-      itr != _extensions.end(); ++itr) {
+  for(std::map<std::string, uint8_t>::const_iterator itr = _extensions.begin(),
+        eoi = _extensions.end(); itr != eoi; ++itr) {
     const std::map<std::string, uint8_t>::value_type& vt = *itr;
     strappend(s, ", ", vt.first, "=", util::uitos(vt.second));
   }
@@ -105,8 +105,8 @@ void HandshakeExtensionMessage::doReceivedAction()
     _peer->port = _tcpPort;
     _peer->setIncomingPeer(false);
   }
-  for(std::map<std::string, uint8_t>::const_iterator itr = _extensions.begin();
-      itr != _extensions.end(); ++itr) {
+  for(std::map<std::string, uint8_t>::const_iterator itr = _extensions.begin(),
+        eoi = _extensions.end(); itr != eoi; ++itr) {
     const std::map<std::string, uint8_t>::value_type& vt = *itr;
     _peer->setExtension(vt.first, vt.second);
   }
@@ -121,7 +121,7 @@ void HandshakeExtensionMessage::doReceivedAction()
   if(_metadataSize > 0) {
     if(attrs.containsKey(bittorrent::METADATA_SIZE)) {
       if(_metadataSize != (size_t)attrs[bittorrent::METADATA_SIZE].i()) {
-	throw DL_ABORT_EX("Wrong metadata_size. Which one is correct!?");
+        throw DL_ABORT_EX("Wrong metadata_size. Which one is correct!?");
       }
     } else {
       attrs[bittorrent::METADATA_SIZE] = _metadataSize;
@@ -130,16 +130,16 @@ void HandshakeExtensionMessage::doReceivedAction()
       _dctx->getOwnerRequestGroup()->initPieceStorage();
       
       SharedHandle<PieceStorage> pieceStorage =
-	_dctx->getOwnerRequestGroup()->getPieceStorage();
+        _dctx->getOwnerRequestGroup()->getPieceStorage();
       pieceStorage->setEndGamePieceNum(0);
     }
   } else if(!attrs.containsKey(bittorrent::METADATA)) {
     throw DL_ABORT_EX("Peer didn't provide metadata_size."
-		      " It seems that it doesn't have whole metadata.");
+                      " It seems that it doesn't have whole metadata.");
   }
 }
 
-void HandshakeExtensionMessage::setPeer(const PeerHandle& peer)
+void HandshakeExtensionMessage::setPeer(const SharedHandle<Peer>& peer)
 {
   _peer = peer;
 }
@@ -160,11 +160,14 @@ HandshakeExtensionMessage::create(const unsigned char* data, size_t length)
   if(length < 1) {
     throw DL_ABORT_EX
       (StringFormat(MSG_TOO_SMALL_PAYLOAD_SIZE,
-		    EXTENSION_NAME.c_str(), length).str());
+                    EXTENSION_NAME.c_str(), length).str());
   }
   HandshakeExtensionMessageHandle msg(new HandshakeExtensionMessage());
-  msg->_logger->debug("Creating HandshakeExtensionMessage from %s",
-		      util::urlencode(data, length).c_str());
+  if(LogFactory::getInstance()->debug()) {
+    LogFactory::getInstance()->debug
+      ("Creating HandshakeExtensionMessage from %s",
+       util::percentEncode(data, length).c_str());
+  }
   const BDE dict = bencode::decode(data+1, length-1);
   if(!dict.isDict()) {
     throw DL_ABORT_EX("Unexpected payload format for extended message handshake");
@@ -179,10 +182,10 @@ HandshakeExtensionMessage::create(const unsigned char* data, size_t length)
   }
   const BDE& extDict = dict["m"];
   if(extDict.isDict()) {
-    for(BDE::Dict::const_iterator i = extDict.dictBegin();
-	i != extDict.dictEnd(); ++i) {
+    for(BDE::Dict::const_iterator i = extDict.dictBegin(),
+          eoi = extDict.dictEnd(); i != eoi; ++i) {
       if((*i).second.isInteger()) {
-	msg->_extensions[(*i).first] = (*i).second.i();
+        msg->_extensions[(*i).first] = (*i).second.i();
       }
     }
   }
