@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -46,7 +46,13 @@
 
 namespace aria2 {
 
-HttpHeaderProcessor::HttpHeaderProcessor():_limit(4096) {}
+HttpHeaderProcessor::HttpHeaderProcessor():
+  _limit(21/*lines*/*8190/*per line*/) {}
+// The above values come from Apache's documentation
+// http://httpd.apache.org/docs/2.2/en/mod/core.html: See
+// LimitRequestFieldSize and LimitRequestLine directive.  Also the
+// page states that the number of request fields rarely exceeds 20.
+// aria2 uses this class in both client and server side.
 
 HttpHeaderProcessor::~HttpHeaderProcessor() {}
 
