@@ -40,6 +40,7 @@
 #include "BitfieldMan.h"
 #include "A2STR.h"
 #include "BtMessageDispatcher.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
@@ -68,7 +69,7 @@ void PeerSessionResource::amChoking(bool b)
 {
   _amChoking = b;
   if(!b) {
-    _lastAmUnchoking.reset();
+    _lastAmUnchoking = global::wallclock;
   }
 }
 
@@ -158,14 +159,14 @@ void PeerSessionResource::fastExtensionEnabled(bool b)
   _fastExtensionEnabled = b;
 }
 
-const std::deque<size_t>& PeerSessionResource::peerAllowedIndexSet() const
+const std::vector<size_t>& PeerSessionResource::peerAllowedIndexSet() const
 {
   return _peerAllowedIndexSet;
 }
 
-static void updateIndexSet(std::deque<size_t>& c, size_t index)
+static void updateIndexSet(std::vector<size_t>& c, size_t index)
 {
-  std::deque<size_t>::iterator i = std::lower_bound(c.begin(), c.end(), index);
+  std::vector<size_t>::iterator i = std::lower_bound(c.begin(), c.end(), index);
   if(i == c.end() || (*i) != index) {
     c.insert(i, index);
   } 
@@ -213,8 +214,8 @@ PeerSessionResource::getExtensionMessageID(const std::string& name) const
 
 std::string PeerSessionResource::getExtensionName(uint8_t id) const
 {
-  for(Extensions::const_iterator itr = _extensions.begin();
-      itr != _extensions.end(); ++itr) {
+  for(Extensions::const_iterator itr = _extensions.begin(),
+        eoi = _extensions.end(); itr != eoi; ++itr) {
     const Extensions::value_type& p = *itr;
     if(p.second == id) {
       return p.first;
@@ -252,7 +253,7 @@ void PeerSessionResource::updateDownloadLength(size_t bytes)
 {
   _peerStat.updateDownloadLength(bytes);
 
-  _lastDownloadUpdate.reset();
+  _lastDownloadUpdate = global::wallclock;
 }
 
 uint64_t PeerSessionResource::getCompletedLength() const
