@@ -38,7 +38,6 @@
 #include <fstream>
 #include <algorithm>
 #include <sstream>
-#include <vector>
 
 #include "RequestGroup.h"
 #include "Option.h"
@@ -62,6 +61,7 @@
 #include "ByteArrayDiskWriter.h"
 #include "a2functional.h"
 #include "ByteArrayDiskWriterFactory.h"
+#include "MetadataInfo.h"
 #ifdef ENABLE_BITTORRENT
 # include "bittorrent_helper.h"
 # include "BtConstants.h"
@@ -77,6 +77,8 @@ const std::set<std::string>& listRequestOptions()
     PREF_CHECK_INTEGRITY,
     PREF_CONTINUE,
     PREF_ALL_PROXY,
+    PREF_ALL_PROXY_USER,
+    PREF_ALL_PROXY_PASSWD,
     PREF_CONNECT_TIMEOUT,
     PREF_DRY_RUN,
     PREF_LOWEST_SPEED_LIMIT,
@@ -93,7 +95,11 @@ const std::set<std::string>& listRequestOptions()
     PREF_HTTP_USER,
     PREF_HTTP_PASSWD,
     PREF_HTTP_PROXY,
+    PREF_HTTP_PROXY_USER,
+    PREF_HTTP_PROXY_PASSWD,
     PREF_HTTPS_PROXY,
+    PREF_HTTPS_PROXY_USER,
+    PREF_HTTPS_PROXY_PASSWD,
     PREF_REFERER,
     PREF_ENABLE_HTTP_KEEP_ALIVE,
     PREF_ENABLE_HTTP_PIPELINING,
@@ -104,10 +110,14 @@ const std::set<std::string>& listRequestOptions()
     PREF_FTP_PASSWD,
     PREF_FTP_PASV,
     PREF_FTP_PROXY,
+    PREF_FTP_PROXY_USER,
+    PREF_FTP_PROXY_PASSWD,
     PREF_FTP_TYPE,
     PREF_FTP_REUSE_CONNECTION,
     PREF_NO_NETRC,
+    PREF_REUSE_URI,
     PREF_SELECT_FILE,
+    PREF_BT_ENABLE_LPD,
     PREF_BT_EXTERNAL_IP,
     PREF_BT_HASH_CHECK_SEED,
     PREF_BT_MAX_OPEN_FILES,
@@ -121,6 +131,8 @@ const std::set<std::string>& listRequestOptions()
     PREF_BT_SEED_UNVERIFIED,
     PREF_BT_STOP_TIMEOUT,
     PREF_BT_TRACKER_INTERVAL,
+    PREF_BT_TRACKER_TIMEOUT,
+    PREF_BT_TRACKER_CONNECT_TIMEOUT,
     PREF_ENABLE_PEER_EXCHANGE,
     PREF_FOLLOW_TORRENT,
     PREF_INDEX_OUT,
@@ -143,31 +155,34 @@ const std::set<std::string>& listRequestOptions()
     PREF_MAX_DOWNLOAD_LIMIT,
     PREF_NO_FILE_ALLOCATION_LIMIT,
     PREF_PARAMETERIZED_URI,
-    PREF_REALTIME_CHUNK_CHECKSUM
+    PREF_REALTIME_CHUNK_CHECKSUM,
+    PREF_REMOVE_CONTROL_FILE,
+    PREF_ALWAYS_RESUME,
+    PREF_MAX_RESUME_FAILURE_TRIES,
+    PREF_HTTP_ACCEPT_GZIP
   };
   static std::set<std::string> requestOptions
-    (&REQUEST_OPTIONS[0],
-     &REQUEST_OPTIONS[arrayLength(REQUEST_OPTIONS)]);;
-
+    (vbegin(REQUEST_OPTIONS), vend(REQUEST_OPTIONS));
   return requestOptions;
 }
 
 static void unfoldURI
-(std::deque<std::string>& result, const std::deque<std::string>& args)
+(std::vector<std::string>& result, const std::vector<std::string>& args)
 {
   ParameterizedStringParser p;
   PStringBuildVisitor v;
-  for(std::deque<std::string>::const_iterator itr = args.begin();
-      itr != args.end(); ++itr) {
+  for(std::vector<std::string>::const_iterator itr = args.begin(),
+        eoi = args.end(); itr != eoi; ++itr) {
     v.reset();
     p.parse(*itr)->accept(v);
     result.insert(result.end(), v.getURIs().begin(), v.getURIs().end()); 
   }
 }
 
-static void splitURI(std::deque<std::string>& result,
-                     std::deque<std::string>::const_iterator begin,
-                     std::deque<std::string>::const_iterator end,
+template<typename InputIterator>
+static void splitURI(std::vector<std::string>& result,
+                     InputIterator begin,
+                     InputIterator end,
                      size_t numSplit)
 {
   size_t numURIs = std::distance(begin, end);
@@ -182,7 +197,7 @@ static void splitURI(std::deque<std::string>& result,
 }
 
 static SharedHandle<RequestGroup> createRequestGroup
-(const SharedHandle<Option>& option, const std::deque<std::string>& uris,
+(const SharedHandle<Option>& option, const std::vector<std::string>& uris,
  bool useOutOption = false)
 {
   SharedHandle<RequestGroup> rg(new RequestGroup(option));
@@ -198,13 +213,23 @@ static SharedHandle<RequestGroup> createRequestGroup
   return rg;
 }
 
+static SharedHandle<MetadataInfo> createMetadataInfo(const std::string& uri)
+{
+  return SharedHandle<MetadataInfo>(new MetadataInfo(uri));
+}
+
+static SharedHandle<MetadataInfo> createMetadataInfoDataOnly()
+{
+  return SharedHandle<MetadataInfo>(new MetadataInfo());
+}
+
 #ifdef ENABLE_BITTORRENT
 
 static
 SharedHandle<RequestGroup>
 createBtRequestGroup(const std::string& torrentFilePath,
                      const SharedHandle<Option>& option,
-                     const std::deque<std::string>& auxUris,
+                     const std::vector<std::string>& auxUris,
                      const std::string& torrentData = "")
 {
   SharedHandle<RequestGroup> rg(new RequestGroup(option));
@@ -212,21 +237,26 @@ createBtRequestGroup(const std::string& torrentFilePath,
   dctx->setDir(option->get(PREF_DIR));
   if(torrentData.empty()) {
     bittorrent::load(torrentFilePath, dctx, auxUris);// may throw exception
+    rg->setMetadataInfo(createMetadataInfo(torrentFilePath));
   } else {
     bittorrent::loadFromMemory(torrentData, dctx, auxUris, "default"); // may
     // throw
     // exception
+    rg->setMetadataInfo(createMetadataInfoDataOnly());
   }
   dctx->setFileFilter(util::parseIntRange(option->get(PREF_SELECT_FILE)));
   std::istringstream indexOutIn(option->get(PREF_INDEX_OUT));
   std::map<size_t, std::string> indexPathMap =
     util::createIndexPathMap(indexOutIn);
-  for(std::map<size_t, std::string>::const_iterator i = indexPathMap.begin();
-      i != indexPathMap.end(); ++i) {
+  for(std::map<size_t, std::string>::const_iterator i = indexPathMap.begin(),
+        eoi = indexPathMap.end(); i != eoi; ++i) {
     dctx->setFilePathWithIndex
       ((*i).first, util::applyDir(dctx->getDir(), (*i).second));
   }
   rg->setDownloadContext(dctx);
+  // Remove "metalink" from Accept Type list to avoid server from
+  // responding Metalink file for web-seeding URIs.
+  util::removeMetalinkContentTypes(rg);
   return rg;
 }
 
@@ -234,7 +264,7 @@ static
 SharedHandle<RequestGroup>
 createBtMagnetRequestGroup(const std::string& magnetLink,
                            const SharedHandle<Option>& option,
-                           const std::deque<std::string>& auxUris)
+                           const std::vector<std::string>& auxUris)
 {
   SharedHandle<RequestGroup> rg(new RequestGroup(option));
   SharedHandle<DownloadContext> dctx
@@ -255,16 +285,17 @@ createBtMagnetRequestGroup(const std::string& magnetLink,
      (new UTMetadataPostDownloadHandler()));
   rg->setDiskWriterFactory
     (SharedHandle<DiskWriterFactory>(new ByteArrayDiskWriterFactory()));
+  rg->setMetadataInfo(createMetadataInfo(magnetLink));
   return rg;
 }
 
 void createRequestGroupForBitTorrent
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
- const std::deque<std::string>& uris,
+ const std::vector<std::string>& uris,
  const std::string& torrentData)
 {
-  std::deque<std::string> nargs;
+  std::vector<std::string> nargs;
   if(option->get(PREF_PARAMETERIZED_URI) == V_TRUE) {
     unfoldURI(nargs, uris);
   } else {
@@ -272,7 +303,7 @@ void createRequestGroupForBitTorrent
   }
   // we ignore -Z option here
   size_t numSplit = option->getAsInt(PREF_SPLIT);
-  std::deque<std::string> auxUris;
+  std::vector<std::string> auxUris;
   splitURI(auxUris, nargs.begin(), nargs.end(), numSplit);
   SharedHandle<RequestGroup> rg =
     createBtRequestGroup(option->get(PREF_TORRENT_FILE), option, auxUris,
@@ -285,7 +316,7 @@ void createRequestGroupForBitTorrent
 
 #ifdef ENABLE_METALINK
 void createRequestGroupForMetalink
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
  const std::string& metalinkData)
 {
@@ -306,12 +337,12 @@ void createRequestGroupForMetalink
 
 class AccRequestGroup {
 private:
-  std::deque<SharedHandle<RequestGroup> >& _requestGroups;
+  std::vector<SharedHandle<RequestGroup> >& _requestGroups;
   ProtocolDetector _detector;
   SharedHandle<Option> _option;
   bool _ignoreLocalPath;
 public:
-  AccRequestGroup(std::deque<SharedHandle<RequestGroup> >& requestGroups,
+  AccRequestGroup(std::vector<SharedHandle<RequestGroup> >& requestGroups,
                   const SharedHandle<Option>& option,
                   bool ignoreLocalPath = false):
     _requestGroups(requestGroups), _option(option),
@@ -321,7 +352,7 @@ public:
   operator()(const std::string& uri)
   {
     if(_detector.isStreamProtocol(uri)) {
-      std::deque<std::string> streamURIs;
+      std::vector<std::string> streamURIs;
       size_t numSplit = _option->getAsInt(PREF_SPLIT);
       for(size_t i = 0; i < numSplit; ++i) {
         streamURIs.push_back(uri);
@@ -335,7 +366,7 @@ public:
     else if(_detector.guessTorrentMagnet(uri)) {
       try {
         SharedHandle<RequestGroup> group =
-          createBtMagnetRequestGroup(uri, _option, std::deque<std::string>());
+          createBtMagnetRequestGroup(uri, _option, std::vector<std::string>());
         _requestGroups.push_back(group);
       } catch(RecoverableException& e) {
         // error occurred while parsing torrent file.
@@ -344,8 +375,8 @@ public:
       }
     } else if(!_ignoreLocalPath && _detector.guessTorrentFile(uri)) {
       try {
-        _requestGroups.push_back(createBtRequestGroup(uri, _option,
-                                                      std::deque<std::string>()));
+        _requestGroups.push_back
+          (createBtRequestGroup(uri, _option, std::vector<std::string>()));
       } catch(RecoverableException& e) {
         // error occurred while parsing torrent file.
         // We simply ignore it. 
@@ -380,13 +411,13 @@ public:
 };
 
 void createRequestGroupForUri
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
- const std::deque<std::string>& uris,
+ const std::vector<std::string>& uris,
  bool ignoreForceSequential,
  bool ignoreLocalPath)
 {
-  std::deque<std::string> nargs;
+  std::vector<std::string> nargs;
   if(option->get(PREF_PARAMETERIZED_URI) == V_TRUE) {
     unfoldURI(nargs, uris);
   } else {
@@ -396,12 +427,12 @@ void createRequestGroupForUri
     std::for_each(nargs.begin(), nargs.end(),
                   AccRequestGroup(result, option, ignoreLocalPath));
   } else {
-    std::deque<std::string>::iterator strmProtoEnd =
+    std::vector<std::string>::iterator strmProtoEnd =
       std::stable_partition(nargs.begin(), nargs.end(), StreamProtocolFilter());
     // let's process http/ftp protocols first.
     if(nargs.begin() != strmProtoEnd) {
       size_t numSplit = option->getAsInt(PREF_SPLIT);
-      std::deque<std::string> streamURIs;
+      std::vector<std::string> streamURIs;
       splitURI(streamURIs, nargs.begin(), strmProtoEnd,
                numSplit);
       SharedHandle<RequestGroup> rg =
@@ -416,13 +447,13 @@ void createRequestGroupForUri
 }
 
 static void createRequestGroupForUriList
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
  std::istream& in)
 {
   UriListParser p(in);
   while(p.hasNext()) {
-    std::deque<std::string> uris;
+    std::vector<std::string> uris;
     SharedHandle<Option> tempOption(new Option());
     p.parseNext(uris, *tempOption.get());
     if(uris.empty()) {
@@ -431,7 +462,8 @@ static void createRequestGroupForUriList
 
     SharedHandle<Option> requestOption(new Option(*option.get()));
     for(std::set<std::string>::const_iterator i =
-          listRequestOptions().begin(); i != listRequestOptions().end(); ++i) {
+          listRequestOptions().begin(), eoi = listRequestOptions().end();
+        i != eoi; ++i) {
       if(tempOption->defined(*i)) {
         requestOption->put(*i, tempOption->get(*i));
       }
@@ -442,7 +474,7 @@ static void createRequestGroupForUriList
 }
 
 void createRequestGroupForUriList
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option)
 {
   if(option->get(PREF_INPUT_FILE) == "-") {
@@ -458,4 +490,19 @@ void createRequestGroupForUriList
   }
 }
 
+SharedHandle<MetadataInfo>
+createMetadataInfoFromFirstFileEntry(const SharedHandle<DownloadContext>& dctx)
+{
+  if(dctx->getFileEntries().empty()) {
+    return SharedHandle<MetadataInfo>();
+  } else {
+    std::vector<std::string> uris;
+    dctx->getFileEntries()[0]->getUris(uris);
+    if(uris.empty()) {
+      return SharedHandle<MetadataInfo>();
+    }
+    return SharedHandle<MetadataInfo>(new MetadataInfo(uris[0]));
+  }
+}
+
 } // namespace aria2
