@@ -75,6 +75,9 @@
 #include "A2STR.h"
 #include "array_fun.h"
 #include "a2functional.h"
+#include "bitfield.h"
+#include "DownloadHandlerConstants.h"
+#include "RequestGroup.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigestHelper.h"
 #endif // ENABLE_MESSAGE_DIGEST
@@ -139,6 +142,29 @@ std::pair<std::string, std::string> split(const std::string& src, const std::str
   return hp;
 }
 
+std::string itos(int64_t value, bool comma)
+{
+  bool flag = false;
+  std::string str;
+  if(value < 0) {
+    if(value == INT64_MIN) {
+      if(comma) {
+        str = "-9,223,372,036,854,775,808";
+      } else {
+        str = "-9223372036854775808";
+      }
+      return str;
+    }
+    flag = true;
+    value = -value;
+  }
+  str = uitos(value, comma);
+  if(flag) {
+    str.insert(str.begin(), '-');
+  }
+  return str;
+}
+
 int64_t difftv(struct timeval tv1, struct timeval tv2) {
   if((tv1.tv_sec < tv2.tv_sec) ||
      ((tv1.tv_sec == tv2.tv_sec) && (tv1.tv_usec < tv2.tv_usec))) {
@@ -216,22 +242,30 @@ bool isHexDigit(const char c)
   return isDigit(c) || ('A' <= c && c <= 'F') || ('a' <= c && c <= 'f');
 }
 
+bool isHexDigit(const std::string& s)
+{
+  for(std::string::const_iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
+    if(!isHexDigit(*i)) {
+      return false;
+    }
+  }
+  return true;
+}
+
 bool inRFC3986ReservedChars(const char c)
 {
   static const char reserved[] = {
     ':' , '/' , '?' , '#' , '[' , ']' , '@',
     '!' , '$' , '&' , '\'' , '(' , ')',
     '*' , '+' , ',' , ';' , '=' };
-  return std::find(&reserved[0], &reserved[arrayLength(reserved)], c) !=
-    &reserved[arrayLength(reserved)];
+  return std::find(vbegin(reserved), vend(reserved), c) != vend(reserved);
 }
 
 bool inRFC3986UnreservedChars(const char c)
 {
   static const char unreserved[] = { '-', '.', '_', '~' };
   return isAlpha(c) || isDigit(c) ||
-    std::find(&unreserved[0], &unreserved[arrayLength(unreserved)], c) !=
-    &unreserved[arrayLength(unreserved)];
+    std::find(vbegin(unreserved), vend(unreserved), c) != vend(unreserved);
 }
 
 bool inRFC2978MIMECharset(const char c)
@@ -242,8 +276,7 @@ bool inRFC2978MIMECharset(const char c)
     '`', '{', '}', '~'
   };
   return isAlpha(c) || isDigit(c) ||
-    std::find(&chars[0], &chars[arrayLength(chars)], c) !=
-    &chars[arrayLength(chars)];
+    std::find(vbegin(chars), vend(chars), c) != vend(chars);
 }
 
 bool inRFC2616HttpToken(const char c)
@@ -253,11 +286,10 @@ bool inRFC2616HttpToken(const char c)
     '^', '_', '`', '|', '~'
   };
   return isAlpha(c) || isDigit(c) ||
-    std::find(&chars[0], &chars[arrayLength(chars)], c) !=
-    &chars[arrayLength(chars)];
+    std::find(vbegin(chars), vend(chars), c) != vend(chars);
 }
 
-std::string urlencode(const unsigned char* target, size_t len) {
+std::string percentEncode(const unsigned char* target, size_t len) {
   std::string dest;
   for(size_t i = 0; i < len; ++i) {
     if(!inRFC3986UnreservedChars(target[i])) {
@@ -269,13 +301,13 @@ std::string urlencode(const unsigned char* target, size_t len) {
   return dest;
 }
 
-std::string urlencode(const std::string& target)
+std::string percentEncode(const std::string& target)
 {
-  return urlencode(reinterpret_cast<const unsigned char*>(target.c_str()),
+  return percentEncode(reinterpret_cast<const unsigned char*>(target.c_str()),
                    target.size());
 }
 
-std::string torrentUrlencode(const unsigned char* target, size_t len) {
+std::string torrentPercentEncode(const unsigned char* target, size_t len) {
   std::string dest;
   for(size_t i = 0; i < len; ++i) {
     if(isAlpha(target[i]) || isDigit(target[i])) {
@@ -287,16 +319,16 @@ std::string torrentUrlencode(const unsigned char* target, size_t len) {
   return dest;
 }
 
-std::string torrentUrlencode(const std::string& target)
+std::string torrentPercentEncode(const std::string& target)
 {
-  return torrentUrlencode
+  return torrentPercentEncode
     (reinterpret_cast<const unsigned char*>(target.c_str()), target.size());
 }
 
-std::string urldecode(const std::string& target) {
+std::string percentDecode(const std::string& target) {
   std::string result;
-  for(std::string::const_iterator itr = target.begin();
-      itr != target.end(); ++itr) {
+  for(std::string::const_iterator itr = target.begin(), eoi = target.end();
+      itr != eoi; ++itr) {
     if(*itr == '%') {
       if(itr+1 != target.end() && itr+2 != target.end() &&
          isHexDigit(*(itr+1)) && isHexDigit(*(itr+2))) {
@@ -313,14 +345,22 @@ std::string urldecode(const std::string& target) {
 }
 
 std::string toHex(const unsigned char* src, size_t len) {
-  char* temp = new char[len*2+1];
-  for(size_t i = 0; i < len; ++i) {
-    sprintf(temp+i*2, "%02x", src[i]);
+  std::string out(len*2, '\0');
+  std::string::iterator o = out.begin();
+  const unsigned char* last = src+len;
+  for(const unsigned char* i = src; i != last; ++i) {
+    *o = (*i >> 4);
+    *(o+1) = (*i)&0x0f;
+    for(int j = 0; j < 2; ++j) {
+      if(*o < 10) {
+        *o += '0';
+      } else {
+        *o += 'a'-10;
+      }
+      ++o;
+    }
   }
-  temp[len*2] = '\0';
-  std::string hex = temp;
-  delete [] temp;
-  return hex;
+  return out;
 }
 
 std::string toHex(const char* src, size_t len)
@@ -421,47 +461,20 @@ int getNum(const char* buf, int offset, size_t length) {
 
 int32_t parseInt(const std::string& s, int32_t base)
 {
-  std::string trimed = trim(s);
-  if(trimed.empty()) {
+  int64_t v = parseLLInt(s, base);
+  if(v < INT32_MIN || INT32_MAX < v) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   "empty string").str());
-  }
-  char* stop;
-  errno = 0;
-  long int v = strtol(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   trimed.c_str()).str());
-  } else if((((v == LONG_MIN) || (v == LONG_MAX)) && (errno == ERANGE)) ||
-            (v > INT32_MAX) ||
-            (v < INT32_MIN)) {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   trimed.c_str()).str());
+                                   s.c_str()).str());
   }
   return v;
 }
 
 uint32_t parseUInt(const std::string& s, int base)
 {
-  std::string trimed = trim(s);
-  if(trimed.empty()) {
+  uint64_t v = parseULLInt(s, base);
+  if(UINT32_MAX < v) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   "empty string").str());
-  }
-  // We don't allow negative number.
-  if(trimed[0] == '-') {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   trimed.c_str()).str());
-  }
-  char* stop;
-  errno = 0;
-  unsigned long int v = strtoul(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   trimed.c_str()).str());
-  } else if(((v == ULONG_MAX) && (errno == ERANGE)) || (v > UINT32_MAX)) {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                                   trimed.c_str()).str());
+                                   s.c_str()).str());
   }
   return v;
 }
@@ -570,7 +583,7 @@ static void computeHeadPieces
     return;
   }
   for(std::vector<SharedHandle<FileEntry> >::const_iterator fi =
-        fileEntries.begin(); fi != fileEntries.end(); ++fi) {
+        fileEntries.begin(), eoi = fileEntries.end(); fi != eoi; ++fi) {
     if((*fi)->getLength() == 0) {
       continue;
     }
@@ -593,7 +606,7 @@ static void computeTailPieces
     return;
   }
   for(std::vector<SharedHandle<FileEntry> >::const_iterator fi =
-        fileEntries.begin(); fi != fileEntries.end(); ++fi) {
+        fileEntries.begin(), eoi = fileEntries.end(); fi != eoi; ++fi) {
     if((*fi)->getLength() == 0) {
       continue;
     }
@@ -616,8 +629,8 @@ void parsePrioritizePieceRange
   std::vector<size_t> indexes;
   std::vector<std::string> parts;
   split(src, std::back_inserter(parts), ",", true);
-  for(std::vector<std::string>::const_iterator i = parts.begin();
-      i != parts.end(); ++i) {
+  for(std::vector<std::string>::const_iterator i = parts.begin(),
+        eoi = parts.end(); i != eoi; ++i) {
     if((*i) == "head") {
       computeHeadPieces(indexes, fileEntries, pieceLength, defaultSize);
     } else if(util::startsWith(*i, "head=")) {
@@ -642,24 +655,13 @@ void parsePrioritizePieceRange
   result.insert(result.end(), indexes.begin(), indexes.end());
 }
 
-static std::string trimBasename(const std::string& src)
-{
-  static const std::string TRIMMED("\r\n\t '\"");
-  std::string fn = File(trim(src, TRIMMED)).getBasename();
-  std::string::iterator enditer = std::remove(fn.begin(), fn.end(), '\\');
-  fn = std::string(fn.begin(), enditer);
-  if(fn == ".." || fn == A2STR::DOT_C) {
-    fn = A2STR::NIL;
-  }
-  return fn;
-}
-
 // Converts ISO/IEC 8859-1 string to UTF-8 string.  If there is a
 // character not in ISO/IEC 8859-1, returns empty string.
 std::string iso8859ToUtf8(const std::string& src)
 {
   std::string dest;
-  for(std::string::const_iterator itr = src.begin(); itr != src.end(); ++itr) {
+  for(std::string::const_iterator itr = src.begin(), eoi = src.end();
+      itr != eoi; ++itr) {
     unsigned char c = *itr;
     if(0xa0 <= c) {
       if(c <= 0xbf) {
@@ -682,14 +684,14 @@ std::string getContentDispositionFilename(const std::string& header)
   std::string filename;
   std::vector<std::string> params;
   split(header, std::back_inserter(params), A2STR::SEMICOLON_C, true);
-  for(std::vector<std::string>::iterator i = params.begin();
-      i != params.end(); ++i) {
-    std::string& param = *i;
+  for(std::vector<std::string>::const_iterator i = params.begin(),
+        eoi = params.end(); i != eoi; ++i) {
+    const std::string& param = *i;
     static const std::string keyName = "filename";
     if(!startsWith(toLower(param), keyName) || param.size() == keyName.size()) {
       continue;
     }
-    std::string::iterator markeritr = param.begin()+keyName.size();
+    std::string::const_iterator markeritr = param.begin()+keyName.size();
     if(*markeritr == '*') {
       // See RFC2231 Section4 and draft-reschke-rfc2231-in-http.
       // Please note that this function doesn't do charset conversion
@@ -710,8 +712,8 @@ std::string getContentDispositionFilename(const std::string& header)
       }
       bool bad = false;
       const std::string& charset = extValues[0];
-      for(std::string::const_iterator j = charset.begin(); j != charset.end();
-          ++j) {
+      for(std::string::const_iterator j = charset.begin(), eoi = charset.end();
+          j != eoi; ++j) {
         // Since we first split parameter by ', we can safely assume
         // that ' is not included in charset.
         if(!inRFC2978MIMECharset(*j)) {
@@ -724,7 +726,8 @@ std::string getContentDispositionFilename(const std::string& header)
       }
       bad = false;
       value = extValues[2];
-      for(std::string::const_iterator j = value.begin(); j != value.end(); ++j){
+      for(std::string::const_iterator j = value.begin(), eoi = value.end();
+          j != eoi; ++j){
         if(*j == '%') {
           if(j+1 != value.end() && isHexDigit(*(j+1)) &&
              j+2 != value.end() && isHexDigit(*(j+2))) {
@@ -743,12 +746,17 @@ std::string getContentDispositionFilename(const std::string& header)
       if(bad) {
         continue;
       }
-      value = trimBasename(urldecode(value));
+      value = percentDecode(value);
       if(toLower(extValues[0]) == "iso-8859-1") {
         value = iso8859ToUtf8(value);
       }
-      filename = value;
-      break;
+      if(!detectDirTraversal(value) &&
+         value.find(A2STR::SLASH_C) == std::string::npos) {
+        filename = value;
+      }
+      if(!filename.empty()) {
+        break;
+      }
     } else {
       for(; markeritr != param.end() && *markeritr == ' '; ++markeritr);
       if(markeritr == param.end() || *markeritr != '=') {
@@ -769,8 +777,14 @@ std::string getContentDispositionFilename(const std::string& header)
       } else {
         filenameLast = value.end();
       }
-      value = trimBasename(urldecode(std::string(value.begin(), filenameLast)));
-      filename = value;
+      static const std::string TRIMMED("\r\n\t '\"");
+      value = percentDecode(std::string(value.begin(), filenameLast));
+      trimSelf(value, TRIMMED);
+      value.erase(std::remove(value.begin(), value.end(), '\\'), value.end());
+      if(!detectDirTraversal(value) &&
+         value.find(A2STR::SLASH_C) == std::string::npos) {
+        filename = value;
+      }
       // continue because there is a chance we can find filename*=...
     }
   }
@@ -936,7 +950,8 @@ bool isNumber(const std::string& what)
   if(what.empty()) {
     return false;
   }
-  for(std::string::const_iterator i = what.begin(); i != what.end(); ++i) {
+  for(std::string::const_iterator i = what.begin(), eoi = what.end();
+      i != eoi; ++i) {
     if(!isDigit(*i)) {
       return false;
     }
@@ -1067,7 +1082,8 @@ getNumericNameInfo(const struct sockaddr* sockaddr, socklen_t len)
 std::string htmlEscape(const std::string& src)
 {
   std::string dest;
-  for(std::string::const_iterator i = src.begin(); i != src.end(); ++i) {
+  for(std::string::const_iterator i = src.begin(), eoi = src.end();
+      i != eoi; ++i) {
     char ch = *i;
     if(ch == '<') {
       dest += "&lt;";
@@ -1131,14 +1147,16 @@ bool saveAs
     return false;
   }
   std::string tempFilename = strconcat(filename, "__temp");
-  std::ofstream out(tempFilename.c_str(), std::ios::binary);
-  if(!out) {
-    return false;
-  }
-  out << data;
-  out.flush();
-  if(!out) {
-    return false;
+  {
+    std::ofstream out(tempFilename.c_str(), std::ios::binary);
+    if(!out) {
+      return false;
+    }
+    out << data;
+    out.flush();
+    if(!out) {
+      return false;
+    }
   }
   return File(tempFilename).renameTo(filename);
 }
@@ -1156,8 +1174,7 @@ std::string applyDir(const std::string& dir, const std::string& relPath)
 
 std::string fixTaintedBasename(const std::string& src)
 {
-  return replace(replace(src, A2STR::SLASH_C, A2STR::UNDERSCORE_C),
-                 A2STR::BACK_SLASH_C, A2STR::UNDERSCORE_C);
+  return escapePath(replace(src, A2STR::SLASH_C, A2STR::UNDERSCORE_C));
 }
 
 void generateRandomKey(unsigned char* key)
@@ -1172,6 +1189,122 @@ void generateRandomKey(unsigned char* key)
 #endif // !ENABLE_MESSAGE_DIGEST
 }
 
+// Returns true is given numeric ipv4addr is in Private Address Space.
+//
+// From Section.3 RFC1918
+// 10.0.0.0        -   10.255.255.255  (10/8 prefix)
+// 172.16.0.0      -   172.31.255.255  (172.16/12 prefix)
+// 192.168.0.0     -   192.168.255.255 (192.168/16 prefix)
+bool inPrivateAddress(const std::string& ipv4addr)
+{
+  if(util::startsWith(ipv4addr, "10.") ||
+     util::startsWith(ipv4addr, "192.168.")) {
+    return true;
+  }
+  if(util::startsWith(ipv4addr, "172.")) {
+    for(int i = 16; i <= 31; ++i) {
+      if(util::startsWith(ipv4addr, "172."+util::itos(i)+".")) {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
+bool detectDirTraversal(const std::string& s)
+{
+  for(std::string::const_iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
+    if(0x00 <= (*i) && (*i) <= 0x1f) {
+      return true;
+    }
+  }
+  return s == A2STR::DOT_C ||
+    s == ".." ||
+    util::startsWith(s, A2STR::SLASH_C) ||
+    util::startsWith(s, "./") ||
+    util::startsWith(s, "../") ||
+    s.find("/../") != std::string::npos ||
+    s.find("/./") != std::string::npos ||
+    util::endsWith(s, "/") ||
+    util::endsWith(s, "/.") ||
+    util::endsWith(s, "/..");
+}
+
+namespace {
+class EscapePath {
+private:
+  char _repChar;
+public:
+  EscapePath(const char& repChar):_repChar(repChar) {}
+
+  char operator()(const char& c) {
+    if(0x00 <= c && c <=0x1f) {
+      return _repChar;
+    }
+#ifdef __MINGW32__
+    // We don't escape '/' because we use it as a path separator.
+    static const char WIN_INVALID_PATH_CHARS[] =
+      { '"', '*', ':', '<', '>', '?', '\\', '|' };
+    if(std::find(vbegin(WIN_INVALID_PATH_CHARS), vend(WIN_INVALID_PATH_CHARS),
+                 c) != vend(WIN_INVALID_PATH_CHARS)) {
+      return _repChar;
+    }
+#endif // __MINGW32__
+    return c;
+  }
+};
+}
+
+std::string escapePath(const std::string& s)
+{
+  std::string d = s;
+  std::transform(d.begin(), d.end(), d.begin(), EscapePath('_'));
+  return d;
+}
+
+bool getCidrPrefix(struct in_addr& in, const std::string& ip, int bits)
+{
+  struct in_addr t;
+  if(inet_aton(ip.c_str(), &t) == 0) {
+    return false;
+  }
+  int lastindex = bits/8;
+  if(lastindex < 4) {
+    char* p = reinterpret_cast<char*>(&t.s_addr);
+    const char* last = p+4;
+    p += lastindex;    
+    if(bits%8 != 0) {
+      *p &= bitfield::lastByteMask(bits);
+      ++p;
+    }
+    for(; p != last; ++p) {
+      *p &= 0;
+    }
+  }
+  in = t;
+  return true;
+}
+
+bool inSameCidrBlock(const std::string& ip1, const std::string& ip2, int bits)
+{
+  struct in_addr in1;
+  struct in_addr in2;
+  if(!getCidrPrefix(in1, ip1, bits) || !getCidrPrefix(in2, ip2, bits)) {
+    return false;
+  }
+  return in1.s_addr == in2.s_addr;
+}
+
+void removeMetalinkContentTypes(const SharedHandle<RequestGroup>& group)
+{
+  for(std::vector<std::string>::const_iterator i =
+	DownloadHandlerConstants::getMetalinkContentTypes().begin(),
+        eoi = DownloadHandlerConstants::getMetalinkContentTypes().end();
+      i != eoi; ++i) {
+    group->removeAcceptType(*i);
+  }
+}
+
 } // namespace util
 
 } // namespace aria2
